@@ -7,7 +7,7 @@ import inspect
 import queue
 import weakref
 from collections import defaultdict
-from threading import Lock
+from threading import Lock, Thread
 from typing import Any, Callable, Generic, cast
 
 from redux.autorun import Autorun
@@ -45,12 +45,11 @@ class Store(Generic[State, Action, Event], SerializationMixin):
     """Redux store for managing state and side effects."""
 
     def __init__(
-        self: Store[State, Action, Event],
+        self: Store,
         reducer: ReducerType[State, Action, Event],
         options: CreateStoreOptions[Action, Event] | None = None,
     ) -> None:
         """Create a new store."""
-        self.finished = False
         self.store_options = options or CreateStoreOptions()
         self.reducer = reducer
         self._create_task = self.store_options.task_creator
@@ -110,23 +109,27 @@ class Store(Generic[State, Action, Event], SerializationMixin):
                 self._create_task(result)
 
     def _run_actions(self: Store[State, Action, Event]) -> None:
-        action = self._actions.pop(0)
-        result = self.reducer(self._state, action)
-        if is_complete_reducer_result(result):
-            self._state = result.state
-            self._call_listeners(self._state)
-            self.dispatch([*(result.actions or []), *(result.events or [])])
-        elif is_state_reducer_result(result):
-            self._state = result
-            self._call_listeners(self._state)
+        while len(self._actions) > 0:
+            action = self._actions.pop(0)
+            if action is not None:
+                result = self.reducer(self._state, action)
+                if is_complete_reducer_result(result):
+                    self._state = result.state
+                    self._call_listeners(self._state)
+                    self.dispatch([*(result.actions or []), *(result.events or [])])
+                elif is_state_reducer_result(result):
+                    self._state = result
+                    self._call_listeners(self._state)
 
-        if isinstance(action, FinishAction):
-            self.dispatch(cast(Event, FinishEvent()))
+                if isinstance(action, FinishAction):
+                    self.dispatch(cast(Event, FinishEvent()))
 
     def _run_event_handlers(self: Store[State, Action, Event]) -> None:
-        event = self._events.pop(0)
-        for event_handler_ in self._event_handlers[type(event)].copy():
-            self._event_handlers_queue.put_nowait((event_handler_, event))
+        while len(self._events) > 0:
+            event = self._events.pop(0)
+            if event is not None:
+                for event_handler in self._event_handlers[type(event)].copy():
+                    self._event_handlers_queue.put_nowait((event_handler, event))
 
     def run(self: Store[State, Action, Event]) -> None:
         """Run the store."""
@@ -137,23 +140,18 @@ class Store(Generic[State, Action, Event], SerializationMixin):
 
                 if len(self._events) > 0:
                     self._run_event_handlers()
-        if (
-            self.finished
-            and self._actions == []
-            and self._events == []
-            and not any(worker.is_alive() for worker in self._workers)
-        ):
-            self.clean_up()
 
     def clean_up(self: Store[State, Action, Event]) -> None:
         """Clean up the store."""
+        self._event_handlers_queue.join()
+        for _ in range(self.store_options.threads):
+            self._event_handlers_queue.put_nowait(None)
+        self._event_handlers_queue.join()
         for worker in self._workers:
             worker.join()
         self._workers.clear()
         self._listeners.clear()
         self._event_handlers.clear()
-        if self.store_options.on_finish:
-            self.store_options.on_finish()
 
     def dispatch(
         self: Store[State, Action, Event],
@@ -175,13 +173,21 @@ class Store(Generic[State, Action, Event], SerializationMixin):
             if isinstance(item, BaseAction):
                 action = cast(Action, item)
                 for action_middleware in self._action_middlewares:
-                    action = action_middleware(action)
-                self._actions.append(action)
+                    action_ = action_middleware(action)
+                    if action_ is None:
+                        break
+                    action = action_
+                else:
+                    self._actions.append(action)
             if isinstance(item, BaseEvent):
                 event = cast(Event, item)
                 for event_middleware in self._event_middlewares:
-                    event = event_middleware(event)
-                self._events.append(event)
+                    event_ = event_middleware(event)
+                    if event_ is None:
+                        break
+                    event = event_
+                else:
+                    self._events.append(event)
 
         if self.store_options.scheduler is None and not self._is_running.locked():
             self.run()
@@ -225,10 +231,25 @@ class Store(Generic[State, Action, Event], SerializationMixin):
 
         return unsubscribe
 
+    def wait_for_store_to_finish(self: Store[State, Action, Event]) -> None:
+        """Wait for the store to finish."""
+        import time
+
+        while True:
+            if (
+                self._actions == []
+                and self._events == []
+                and self._event_handlers_queue.qsize() == 0
+            ):
+                time.sleep(self.store_options.grace_time_in_seconds)
+                self.clean_up()
+                if self.store_options.on_finish:
+                    self.store_options.on_finish()
+                break
+            time.sleep(0.1)
+
     def _handle_finish_event(self: Store[State, Action, Event]) -> None:
-        for _ in range(self.store_options.threads):
-            self._event_handlers_queue.put_nowait(None)
-        self.finished = True
+        Thread(target=self.wait_for_store_to_finish).start()
 
     def autorun(
         self: Store[State, Action, Event],
