@@ -39,7 +39,7 @@ class StoreSnapshot:
         self.results_dir = path.parent / 'results' / file / test_id.split('::')[-1][5:]
         if self.results_dir.exists():
             for file in self.results_dir.glob(
-                'store-*' if override else 'store-*.mismatch.json',
+                'store-*.jsonc' if override else 'store-*.mismatch.jsonc',
             ):
                 file.unlink()  # pragma: no cover
         self.results_dir.mkdir(parents=True, exist_ok=True)
@@ -51,7 +51,12 @@ class StoreSnapshot:
     def json_snapshot(self: StoreSnapshot) -> str:
         """Return the snapshot of the current state of the store."""
         return (
-            json.dumps(self.store.snapshot, indent=2, sort_keys=True)
+            json.dumps(
+                self.store.snapshot,
+                indent=2,
+                sort_keys=True,
+                ensure_ascii=False,
+            )
             if self.store._state  # noqa: SLF001
             else ''
         )
@@ -87,12 +92,7 @@ class StoreSnapshot:
                 mismatch_path.write_text(  # pragma: no cover
                     f'// MISMATCH: {filename}\n{new_snapshot}\n',
                 )
-            if title:
-                assert (
-                    new_snapshot == old_snapshot
-                ), f'Store snapshot mismatch for {title}'
-            else:
-                assert new_snapshot == old_snapshot, 'Store snapshot mismatch'
+            assert new_snapshot == old_snapshot, f'Store snapshot mismatch - {filename}'
 
         self.test_counter[title] += 1
 
